/*
*
* mod_gfx.h - A support header for mod_gfx
* Copyright (C) 2009 Michael Spiegle (mike@nauticaltech.com)
* 
* This file is part of mod_gfx.
* 
* mod_gfx is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* mod_gfx is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*/

#ifndef _MOD_GFX_H_
#define _MOD_GFX_H_

#include <apr_errno.h>
#include <apr_pools.h>
#include <httpd.h>
#include <http_config.h>
#include <apr_hash.h>
#include <gd.h>

#define MAX_LEN 2048
#define CON_TIMEOUT -1
#define BUF_SIZE 4096

typedef enum {
    HTTP_PROTO_BEGIN = 0x10,
    HTTP_PROTO_VERSION,
    HTTP_PROTO_STATUS,
    HTTP_PROTO_MSG,
    HTTP_PROTO_KEY,
    HTTP_PROTO_VALUE,
    HTTP_PROTO_HFIN,
    HTTP_PROTO_BODY,
    HTTP_PROTO_END
} http_state_t;

typedef enum {
	IMAGE_TYPE_JPG = 0x20,
	IMAGE_TYPE_PNG,
	IMAGE_TYPE_GIF,
	IMAGE_TYPE_SRC
} image_type_t;

typedef enum {
	IMAGE_ACTION_RESIZE = 0x40,
	IMAGE_ACTION_RESAMPLE,
	IMAGE_ACTION_CROP,
	IMAGE_ACTION_WATERMARK,
	IMAGE_ACTION_NOOP
} image_action_t;

typedef struct {
	char* name;
	//we're going to disable multiple actions for now
	//apr_array_header_t* actions;
	image_action_t action;
	apr_int64_t rsx;
	apr_int64_t rsy;
	apr_int64_t locx;
	apr_int64_t locy;
	apr_int64_t mergepct;
	gdImagePtr watermark;
	apr_int64_t qual;
	image_type_t output_format;
} gfx_profile_config_t;

typedef struct {
	char* name;
	ap_regex_t*	regex;
	char skippath;
	apr_array_header_t* hosts;
} gfx_origin_config_t;

typedef struct {
} gfx_container_config_t;

typedef struct {
	apr_hash_t* profiles;
	apr_hash_t* origins;
} gfx_server_config_t;

const char IMAGE_SIG_JPG[] = { (char)0xFF, (char)0xD8, (char)0xFF };
const char IMAGE_SIG_GIF[] = { (char)0x47, (char)0x49, (char)0x46 };
const char IMAGE_SIG_PNG[] = { (char)0x89, (char)0x50, (char)0x4E, (char)0x47,
                               (char)0x0D, (char)0x0A, (char)0x1A, (char)0x0A };

#define IMAGE_SIG_SIZE_JPG 3
#define IMAGE_SIG_SIZE_GIF 3
#define IMAGE_SIG_SIZE_PNG 8

module AP_MODULE_DECLARE_DATA gfx_module;

//static int gfx_header_printer(void*, const char*, const char*);
static apr_status_t gfx_pool_cleanup(void*);
static apr_status_t gfx_pool_cleanup_gd(void*);
static int gfx_image_type(void*);
//static char* gfx_output_format(image_type_t);
gdImagePtr gfx_process_image(gdImagePtr src, gfx_profile_config_t* profile);
unsigned int gfx_custom_response(request_rec* r, char* message, unsigned int rc);
gdImagePtr gfx_gd_from_blob(void* b, apr_size_t bp, image_type_t* image_type);

//REALLY unhappy about this.  If I remove them, my compiler complains.
//I would be happy to fix it if anyone has any bright ideas.
ap_conf_vector_t* ap_create_per_dir_config(apr_pool_t*);
void* ap_set_config_vectors(server_rec*, ap_conf_vector_t*, const char*, module*, apr_pool_t*);

#endif
